@@ -1,4 +1,4 @@
-#! /usr/bin/perl -w
+#! /usr/bin/perl
 
 # debchange: update the debian changelog using your favorite visual editor
 # For options, see the usage message below.
@@ -13,7 +13,7 @@
 # Originally by Christoph Lameter <clameter@debian.org>
 # Modified extensively by Julian Gilbey <jdg@debian.org>
 #
-# Copyright 1999-2005 by Julian Gilbey 
+# Copyright 1999-2005 by Julian Gilbey
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,14 +26,15 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 use 5.008;  # We're using PerlIO layers
 use strict;
+use warnings;
 use open ':utf8';  # changelogs are written with UTF-8 encoding
 use filetest 'access';  # use access rather than stat for -w
-use Encode 'decode_utf8';  # for checking whether user names are valid
+# for checking whether user names are valid and making format() behave
+use Encode qw/decode_utf8 encode_utf8/;
 use Getopt::Long;
 use File::Copy;
 use File::Basename;
@@ -60,11 +61,11 @@ sub have_lpdc {
     };
 
     if ($@) {
-        if ($@ =~ m%^Can\'t locate Parse/DebControl%) {
-            $lpdc_broken="the libparse-debcontrol-perl package is not installed";
-        } else {
-            $lpdc_broken="couldn't load Parse::DebControl: $@";
-        }
+	if ($@ =~ m%^Can\'t locate Parse/DebControl%) {
+	    $lpdc_broken="the libparse-debcontrol-perl package is not installed";
+	} else {
+	    $lpdc_broken="couldn't load Parse::DebControl: $@";
+	}
     }
     else { $lpdc_broken=''; }
     return $lpdc_broken ? 0 : 1;
@@ -89,8 +90,9 @@ Options:
          specified by --distribution).
   --force-save-on-release
          When --release is used and an editor opened to allow inspection
-         of the changelog, force the file to be saved in the editor even
-         if no further changes are made. (default)
+         of the changelog, require the user to save the changelog their
+         editor opened.  Otherwise, the original changelog will not be
+         modified. (default)
   --no-force-save-on-release
          Do not do so. Note that a dummy changelog entry made be supplied
          in order to achieve the same effect - e.g. $progname --release ""
@@ -121,17 +123,19 @@ Options:
          Increment the Debian release number for a Debian Security Team upload
   -U, --upstream
          Increment the Debian release number without any distribution name
+  --team
+         Increment the Debian release number for a team upload
   --bpo
          Increment the Debian release number for a Backports.org upload
-	 to "lenny-backports"
+	 to "squeeze-backports"
   -l, --local <suffix>
          Add a suffix to the Debian version number for a local build
   -b, --force-bad-version
          Force a version to be less than the current one (e.g., when
          backporting)
-  --allow-lower-version
+  --allow-lower-version <pattern>
          Allow a version to be less than the current one (e.g., when
-         backporting) if it matches a specified pattern
+         backporting) if it matches the specified pattern
   --force-distribution
          Force the provided distribution to be used, even if it doesn't match
          the list of known distributions
@@ -147,7 +151,7 @@ Options:
   --no-preserve
          Do not preserve the directory name (default)
   --distributor <distributor>
-         Override the distributor ID from lsb_release.
+         Override the distributor ID from dpkg-vendor.
   -D, --distribution <dist>
          Use the specified distribution in the changelog entry being edited
   -u, --urgency <urgency>
@@ -160,8 +164,14 @@ Options:
   --[no]multimaint
          When appending an entry to a changelog section (-a), [do not]
          indicate if multiple maintainers are now involved (default: do so)
+  --[no]multimaint-merge
+         When appending an entry to a changelog section, [do not] merge the
+         entry into an existing changelog section for the current author.
+         (default: do not)
   -m, --maintmaint
          Don\'t change (maintain) the maintainer details in the changelog entry
+  -M, --controlmaint
+         Use maintainer name and email from the debian/control Maintainer field
   -t, --mainttrailer
          Don\'t change (maintain) the trailer line in the changelog entry; i.e.
          maintain the maintainer and date/time details
@@ -174,7 +184,7 @@ Options:
          What constitutes a matching directory name; REGEX is
          a Perl regular expression; the string \`PACKAGE\' will
          be replaced by the package name; see manpage for details
-         (default: 'PACKAGE(-.*)?')
+         (default: 'PACKAGE(-.+)?')
   --no-conf, --noconf
          Don\'t read devscripts config files; must be the first option given
   --release-heuristic log|changelog
@@ -184,7 +194,7 @@ Options:
          Display this help message and exit
   --version
          Display version information
-  At most one of -a, -i, -e, -r, -v, -d, -n, --bin-nmu, -R, -q, --qa, -s, --bpo, -l
+  At most one of -a, -i, -e, -r, -v, -d, -n, --bin-nmu, -R, -q, --qa, -s, --team, --bpo, -l
   (or their long equivalents) may be used.
   With no options, one of -i or -a is chosen by looking for a .upload
   file in the parent directory and checking its contents.
@@ -207,9 +217,10 @@ EOF
 
 # Start by setting default values
 my $check_dirname_level = 1;
-my $check_dirname_regex = 'PACKAGE(-.*)?';
+my $check_dirname_regex = 'PACKAGE(-.+)?';
 my $opt_p = 0;
 my $opt_query = 1;
+my $opt_distributor = undef;
 my $opt_release_heuristic = 'log';
 my $opt_multimaint = 1;
 my $opt_multimaint_merge = 0;
@@ -231,7 +242,7 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
 		       'DEBCHANGE_PRESERVE' => 'no',
 		       'DEBCHANGE_QUERY_BTS' => 'yes',
 		       'DEVSCRIPTS_CHECK_DIRNAME_LEVEL' => 1,
-		       'DEVSCRIPTS_CHECK_DIRNAME_REGEX' => 'PACKAGE(-.*)?',
+		       'DEVSCRIPTS_CHECK_DIRNAME_REGEX' => 'PACKAGE(-.+)?',
 		       'DEBCHANGE_RELEASE_HEURISTIC' => 'log',
 		       'DEBCHANGE_MULTIMAINT' => 'yes',
 		       'DEBCHANGE_TZ' => $ENV{TZ}, # undef if TZ unset
@@ -240,10 +251,11 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
 		       'DEBCHANGE_LOWER_VERSION_PATTERN' => '',
 		       'DEBCHANGE_AUTO_NMU' => 'yes',
 		       'DEBCHANGE_FORCE_SAVE_ON_RELEASE' => 'yes',
+		       'DEBCHANGE_DISTRIBUTOR' => '',
 		       );
     $config_vars{'DEBCHANGE_TZ'} ||= '';
     my %config_default = %config_vars;
-    
+
     my $shell_cmd;
     # Set defaults
     foreach my $var (keys %config_vars) {
@@ -283,6 +295,7 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
     chomp $modified_conf_msg;
 
     $opt_p = $config_vars{'DEBCHANGE_PRESERVE'} eq 'yes' ? 1 : 0;
+    $opt_distributor = $config_vars{'DEBCHANGE_DISTRIBUTOR'};
     $opt_query = $config_vars{'DEBCHANGE_QUERY_BTS'} eq 'no' ? 0 : 1;
     $check_dirname_level = $config_vars{'DEVSCRIPTS_CHECK_DIRNAME_LEVEL'};
     $check_dirname_regex = $config_vars{'DEVSCRIPTS_CHECK_DIRNAME_REGEX'};
@@ -302,10 +315,9 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
 # with older debchange versions.
 my ($opt_help, $opt_version);
 my ($opt_i, $opt_a, $opt_e, $opt_r, $opt_v, $opt_b, $opt_d, $opt_D, $opt_u, $opt_force_dist);
-my ($opt_n, $opt_bn, $opt_R, $opt_qa, $opt_s, $opt_U, $opt_bpo, $opt_l, $opt_c, $opt_m, $opt_create, $opt_package, @closes);
+my ($opt_n, $opt_bn, $opt_R, $opt_qa, $opt_s, $opt_team, $opt_U, $opt_bpo, $opt_l, $opt_c, $opt_m, $opt_M, $opt_create, $opt_package, @closes);
 my ($opt_news);
-my ($opt_ignore, $opt_level, $opt_regex, $opt_noconf, $opt_empty);
-my ($opt_distributor);
+my ($opt_level, $opt_regex, $opt_noconf, $opt_empty);
 
 Getopt::Long::Configure('bundling');
 GetOptions("help|h" => \$opt_help,
@@ -318,7 +330,7 @@ GetOptions("help|h" => \$opt_help,
 	   "package=s" => \$opt_package,
 	   "v|newversion=s" => \$opt_v,
 	   "b|force-bad-version" => \$opt_b,
-	   "allow-lower-version" => \$opt_allow_lower,
+	   "allow-lower-version=s" => \$opt_allow_lower,
 	   "force-distribution" => \$opt_force_dist,
 	   "d|fromdirname" => \$opt_d,
 	   "p" => \$opt_p,
@@ -332,6 +344,7 @@ GetOptions("help|h" => \$opt_help,
 	   "q|qa" => \$opt_qa,
 	   "s|security" => \$opt_s,
 	   "U|upstream" => \$opt_U,
+	   "team" => \$opt_team,
 	   "bpo" => \$opt_bpo,
 	   "l|local=s" => \$opt_l,
 	   "query!" => \$opt_query,
@@ -340,9 +353,11 @@ GetOptions("help|h" => \$opt_help,
 	   "news:s" => \$opt_news,
 	   "multimaint!" => \$opt_multimaint,
 	   "multi-maint!" => \$opt_multimaint,
+	   'multimaint-merge!' => \$opt_multimaint_merge,
+	   'multi-maint-merge!' => \$opt_multimaint_merge,
 	   "m|maintmaint" => \$opt_m,
+	   "M|controlmaint" => \$opt_M,
 	   "t|mainttrailer!" => \$opt_t,
-	   "ignore-dirname" => \$opt_ignore,
 	   "check-dirname-level=s" => \$opt_level,
 	   "check-dirname-regex=s" => \$opt_regex,
 	   "noconf" => \$opt_noconf,
@@ -371,11 +386,6 @@ if ($opt_noconf) {
 if ($opt_help) { usage; exit 0; }
 if ($opt_version) { version; exit 0; }
 
-# dirname stuff
-if ($opt_ignore) {
-    fatal "--ignore-dirname has been replaced by --check-dirname-level and\n--check-dirname-regex; run $progname --help for more details";
-}
-
 if (defined $opt_level) {
     if ($opt_level =~ /^[012]$/) { $check_dirname_level = $opt_level; }
     else {
@@ -386,8 +396,8 @@ if (defined $opt_level) {
 if (defined $opt_regex) { $check_dirname_regex = $opt_regex; }
 
 # Only allow at most one non-help option
-fatal "Only one of -a, -i, -e, -r, -v, -d, -n/--nmu, --bin-nmu, -R/--rebuild, -q/--qa, -s/--security, --bpo, -l/--local is allowed;\ntry $progname --help for more help"
-    if ($opt_i?1:0) + ($opt_a?1:0) + ($opt_e?1:0) + ($opt_r?1:0) + ($opt_v?1:0) + ($opt_d?1:0) + ($opt_n?1:0) + ($opt_bn?1:0) + ($opt_R?1:0) + ($opt_qa?1:0) + ($opt_s?1:0) + ($opt_bpo?1:0) + ($opt_l?1:0) > 1;
+fatal "Only one of -a, -i, -e, -r, -v, -d, -n/--nmu, --bin-nmu, -R/--rebuild, -q/--qa, -s/--security, --team, --bpo, -l/--local is allowed;\ntry $progname --help for more help"
+    if ($opt_i?1:0) + ($opt_a?1:0) + ($opt_e?1:0) + ($opt_r?1:0) + ($opt_v?1:0) + ($opt_d?1:0) + ($opt_n?1:0) + ($opt_bn?1:0) + ($opt_R?1:0) + ($opt_qa?1:0) + ($opt_s?1:0) + ($opt_team?1:0) + ($opt_bpo?1:0) + ($opt_l?1:0) > 1;
 
 if ($opt_s) {
     $opt_u = "high";
@@ -400,10 +410,10 @@ if (defined $opt_u) {
 
 # See if we're Debian, Ubuntu or someone else, if we can
 my $distributor;
-if (defined $opt_distributor) {
+if (not $opt_distributor eq '') {
     $distributor = $opt_distributor;
-} elsif (system('command -v lsb_release >/dev/null 2>&1') >> 8 == 0) {
-    $distributor = `lsb_release -is 2>/dev/null`;
+} elsif (system('command -v dpkg-vendor >/dev/null 2>&1') >> 8 == 0) {
+    $distributor = `dpkg-vendor --query Vendor 2>/dev/null`;
     chomp $distributor;
 }
 $distributor ||= 'Debian';
@@ -414,13 +424,13 @@ if ($distributor eq 'Ubuntu' and ($opt_n or $opt_bn or $opt_qa or $opt_bpo)) {
 # Check the distro name given.
 if (defined $opt_D) {
     if ($distributor eq 'Debian') {
-	unless ($opt_D =~ /^(unstable|((old)?stable|testing)(-security)?|experimental|UNRELEASED|(sarge|etch|lenny)(-volatile|-backports)|((oldstable|testing)-)?proposed-updates)$/) {
-	    warn "$progname warning: Recognised distributions are: unstable, testing, stable,\noldstable, experimental, UNRELEASED, {sarge,etch,lenny}-{volatile,backports},\n{oldstable-,testing-,}proposed-updates and {testing,stable,oldstable}-security.\nUsing your request anyway.\n";
+	unless ($opt_D =~ /^(unstable|((old)?stable|testing)(-security)?|experimental|UNRELEASED|lenny-volatile|lenny-backports(-sloppy)?|squeeze-backports|((oldstable|testing)-)?proposed-updates)$/) {
+	    warn "$progname warning: Recognised distributions are: unstable, testing, stable,\noldstable, experimental, UNRELEASED, lenny-volatile, lenny-backports{-sloppy,}, squeeze-backports,\n{oldstable-,testing-,}proposed-updates and {testing,stable,oldstable}-security.\nUsing your request anyway.\n";
 	    $warnings++ if not $opt_force_dist;
 	}
     } elsif ($distributor eq 'Ubuntu') {
-	unless ($opt_D =~ /^((dapper|gutsy|hardy|intrepid|jaunty|karmic)(-updates|-security|-proposed|-backports)?|UNRELEASED)$/) {
-	    warn "$progname warning: Recognised distributions are:\n{dapper,gutsy,hardy,intrepid,jaunty,karmic}{,-updates,-security,-proposed,-backports} and UNRELEASED.\nUsing your request anyway.\n";
+	unless ($opt_D =~ /^((hardy|lucid|maverick|natty|oneiric|precise)(-updates|-security|-proposed|-backports)?|UNRELEASED)$/) {
+	    warn "$progname warning: Recognised distributions are:\n{hardy,lucid,maverick,natty,oneiric,precise}{,-updates,-security,-proposed,-backports} and UNRELEASED.\nUsing your request anyway.\n";
 	    $warnings++ if not $opt_force_dist;
 	}
     } else {
@@ -430,9 +440,10 @@ if (defined $opt_D) {
 
 fatal "--closes should not be used with --news; put bug numbers in the changelog not the NEWS file"
     if $opt_news && @closes;
-    
-fatal "--package can only be used with --create"
-    if $opt_package && ! $opt_create;
+
+# hm, this can probably be used with more than just -i.
+fatal "--package can only be used with --create, --increment and --newversion"
+    if $opt_package && ! ($opt_create || $opt_i || $opt_v);
 
 my $changelog_path = $opt_c || $ENV{'CHANGELOG'} || 'debian/changelog';
 my $real_changelog_path = $changelog_path;
@@ -447,8 +458,8 @@ fatal "--package cannot be used when creating a NEWS file"
 
 if ($opt_create) {
     if ($opt_a || $opt_i || $opt_e || $opt_r || $opt_b || $opt_n || $opt_bn ||
-	    $opt_R || $opt_qa || $opt_s || $opt_bpo || $opt_l || $opt_allow_lower) {
-	warn "$progname warning: ignoring -a/-i/-e/-r/-b/--allow-lower-version/-n/--bin-nmu/-R/-q/--qa/-s/--bpo/-l options with --create\n";
+	    $opt_R || $opt_qa || $opt_s || $opt_team || $opt_bpo || $opt_l || $opt_allow_lower) {
+	warn "$progname warning: ignoring -a/-i/-e/-r/-b/--allow-lower-version/-n/--bin-nmu/-R/-q/--qa/-s/--team/--bpo/-l options with --create\n";
 	$warnings++;
     }
     if ($opt_package && $opt_d) {
@@ -473,7 +484,7 @@ if (! $opt_create) {
 		fatal "Cannot find $changelog_path anywhere!\nAre you in the source code tree?\n(You could use --create if you wish to create this file.)";
 	    }
 	}
-	
+
 	# Can't write, so stop now.
 	if (! -w $changelog_path) {
 	    fatal "$changelog_path is not writable!";
@@ -515,8 +526,8 @@ my $MAINTAINER = 'MAINTAINER';
 my $EMAIL = 'EMAIL';
 my $DISTRIBUTION = 'UNRELEASED';
 my $bpo_dist = '';
-my %bpo_dists = ( '40', 'etch', '50', 'lenny' );
-my $latest_bpo_dist = '50';
+my %bpo_dists = ( 50, 'lenny', 60, 'squeeze' );
+my $latest_bpo_dist = '60';
 my $CHANGES = '';
 # Changelog urgency, possibly propogated to NEWS files
 my $CL_URGENCY = '';
@@ -562,8 +573,8 @@ if (! $opt_create || ($opt_create && $opt_news)) {
     fatal "No distribution in changelog!"
 	unless exists $changelog{'Distribution'};
     if ($distributor eq 'Ubuntu') {
-	# In Ubuntu uploads should go to karmic
-	$DISTRIBUTION = 'karmic';
+	# In Ubuntu uploads should go to precise
+	$DISTRIBUTION = 'oneiric';
     } else {
 	$DISTRIBUTION=$changelog{'Distribution'};
     }
@@ -604,7 +615,7 @@ if (! $opt_create || ($opt_create && $opt_news)) {
 	my $gooddir;
 	if ($re =~ m%/%) { $gooddir = eval "cwd() =~ /^$re\$/;"; }
 	else { $gooddir = eval "basename(cwd()) =~ /^$re\$/;"; }
-	
+
 	if (! $gooddir) {
 	    my $pwd = cwd();
 	    fatal <<"EOF";
@@ -633,21 +644,22 @@ EOF
 	    # don't know anything
 	}
     }
-    if ($opt_package) {
-	if ($opt_package =~ m/^[a-z0-9][a-z0-9+\-\.]+$/) {
-	    $PACKAGE=$opt_package;
-	} else {
-	    warn "$progname warning: illegal package name used with --package: $opt_package\n";
-	    $warnings++;
-	}
-    }
     if ($opt_v) {
 	$VERSION=$opt_v;
     }
     if ($opt_D) {
 	$DISTRIBUTION=$opt_D;
     }
-}    
+}
+
+if ($opt_package) {
+    if ($opt_package =~ m/^[a-z0-9][a-z0-9+\-\.]+$/) {
+	$PACKAGE=$opt_package;
+    } else {
+	warn "$progname warning: illegal package name used with --package: $opt_package\n";
+	$warnings++;
+    }
+}
 
 # Clean up after old versions of debchange
 if (-f "debian/RELEASED") {
@@ -684,7 +696,7 @@ if (! exists $env{'DEBEMAIL'} or ! exists $env{'DEBFULLNAME'}) {
 }
 
 # Now use the gleaned values to detemine our MAINTAINER and EMAIL values
-if (! $opt_m) {
+if (! $opt_m and ! $opt_M) {
     if (exists $env{'DEBFULLNAME'}) {
 	$MAINTAINER = $env{'DEBFULLNAME'};
     } elsif (exists $env{'NAME'}) {
@@ -729,13 +741,33 @@ if (! $opt_m) {
 	$EMAIL = $addr if $addr;
     }
     # Otherwise, $EMAIL retains its default value of the last changelog entry
-} # if (! $opt_m)
+} # if (! $opt_m and ! $opt_M)
+
+if ($opt_M) {
+    if (-f 'debian/control') {
+	if (have_lpdc()) {
+	    my $parser = Parse::DebControl->new;
+	    my $deb822 = $parser->parse_file('debian/control', {stripComments => 'true'});
+	    my $maintainer = decode_utf8($deb822->[0]->{'Maintainer'});
+	    if ($maintainer =~ /^(.*)\s+<(.*)>$/) {
+		$MAINTAINER = $1;
+		$EMAIL = $2;
+	    } else {
+		fatal "$progname: invalid debian/control Maintainer field value\n";
+	    }
+	} else {
+	    fatal "$progname: unable to get maintainer from debian/control: $lpdc_broken\n";
+	}
+    } else {
+	fatal "Missing file debian/control";
+    }
+}
 
 #####
 
-if ($opt_auto_nmu eq 'yes' and ! $opt_v and ! $opt_l and ! $opt_s and 
-    ! $opt_qa and ! $opt_bpo and ! $opt_bn and ! $opt_R and ! $opt_n and ! $opt_c and
-    ! (exists $ENV{'CHANGELOG'} and length $ENV{'CHANGELOG'}) and
+if ($opt_auto_nmu eq 'yes' and ! $opt_v and ! $opt_l and ! $opt_s and
+    ! $opt_team and ! $opt_qa and ! $opt_bpo and ! $opt_bn and ! $opt_R and ! $opt_n and ! $opt_c and
+    ! (exists $ENV{'CHANGELOG'} and length $ENV{'CHANGELOG'}) and ! $opt_M and
     ! $opt_create and ! $opt_a_passed and ! $opt_r and ! $opt_e and
     $distributor ne 'Ubuntu' and
     ! ($opt_release_heuristic eq 'changelog' and
@@ -752,7 +784,7 @@ if ($opt_auto_nmu eq 'yes' and ! $opt_v and ! $opt_l and ! $opt_s and
 	    my $packager = "$MAINTAINER <$EMAIL>";
 
 	    if (! grep { $_ eq $packager } ($maintainer, @uploaders) and
-		$packager ne $changelog{'Maintainer'}) {
+		$packager ne $changelog{'Maintainer'} and ! $opt_team) {
 		$opt_n=1;
 		$opt_a=0;
 	    }
@@ -869,7 +901,7 @@ if ($opt_news && !$opt_i && !$opt_a) {
 
 # Are we going to have to figure things out for ourselves?
 if (! $opt_i && ! $opt_v && ! $opt_d && ! $opt_a && ! $opt_e && ! $opt_r &&
-    ! $opt_n && ! $opt_bn && ! $opt_R && ! $opt_qa && ! $opt_s && ! $opt_bpo &&
+    ! $opt_n && ! $opt_bn && ! $opt_R && ! $opt_qa && ! $opt_s && ! $opt_team && ! $opt_bpo &&
     ! $opt_l && ! $opt_create) {
     # Yes, we are
     if ($opt_release_heuristic eq 'log') {
@@ -946,7 +978,7 @@ my $line;
 my $optionsok=0;
 my $merge=0;
 
-if (($opt_i || $opt_n || $opt_bn || $opt_R || $opt_qa || $opt_s || $opt_bpo || $opt_l || $opt_v || $opt_d ||
+if (($opt_i || $opt_n || $opt_bn || $opt_R || $opt_qa || $opt_s || $opt_team || $opt_bpo || $opt_l || $opt_v || $opt_d ||
     ($opt_news && $VERSION ne $changelog{'Version'})) && ! $opt_create) {
 
     $optionsok=1;
@@ -990,7 +1022,7 @@ if (($opt_i || $opt_n || $opt_bn || $opt_R || $opt_qa || $opt_s || $opt_bpo || $
 	    $opt_p=1;
 	}
 
-	if (system("dpkg --compare-versions $VERSION lt $NEW_VERSION" .
+	if (system("dpkg --compare-versions $VERSION le $NEW_VERSION" .
 		  " 2>/dev/null 1>&2")) {
 	    if ($opt_b or ($opt_allow_lower and $NEW_VERSION =~ /$opt_allow_lower/)) {
 		warn "$progname warning: new version ($NEW_VERSION) is less than\n" .
@@ -1012,7 +1044,7 @@ if (($opt_i || $opt_n || $opt_bn || $opt_R || $opt_qa || $opt_s || $opt_bpo || $
     # including epochs.
 
     if (! $NEW_VERSION) {
-	if ($VERSION =~ /(.*?)([a-yA-Y][a-zA-Z]*|\d+)(~)?$/i) {
+	if ($VERSION =~ /(.*?)([a-yA-Y][a-zA-Z]*|\d+)([+~])?$/i) {
 	    my $extra=$3 || '';
 	    my $useextra = 0;
 	    my $end=$2;
@@ -1020,13 +1052,7 @@ if (($opt_i || $opt_n || $opt_bn || $opt_R || $opt_qa || $opt_s || $opt_bpo || $
 	    # If it's not already an NMU make it so
 	    # otherwise we can be safe if we behave like dch -i
 
-	    if ($distributor eq 'Ubuntu' and $start !~ /(build|ubuntu|prop(osed)?)(\d+\.)*$/ and not $opt_U and not $opt_R) {
-		if ($start =~ /~ppa([0-9]*)$/) {
-		    $end++;
-		} else {
-		    $end .= "ubuntu1";
-		}
-	    } elsif (($opt_n or $opt_s) and (
+	    if (($opt_n or $opt_s) and $distributor ne 'Ubuntu' and (
 		($VERSION eq $UVERSION and not $start =~ /\+nmu/)
 		or ($VERSION ne $UVERSION and not $start =~ /\.$/))) {
 
@@ -1034,19 +1060,19 @@ if (($opt_i || $opt_n || $opt_bn || $opt_R || $opt_qa || $opt_s || $opt_bpo || $
 		    # First NMU of a Debian native package
 		    $end .= "+nmu1";
 		} else {
-	    	    $end += 0.1;
+		    $end += 0.1;
 		}
-            } elsif ($opt_bn and not $start =~ /\+b/) {
-                $end .= "+b1";
-            } elsif ($opt_R and not $start =~ /build/ and not $start =~ /ubuntu/) {
-                $end .= "build1";
+	    } elsif ($opt_bn and not $start =~ /\+b/) {
+		$end .= "+b1";
+	    } elsif ($opt_R and not $start =~ /build/ and not $start =~ /ubuntu/) {
+		$end .= "build1";
 	    } elsif ($opt_qa and $start =~/(.*?)-(\d+)\.$/) {
-                # Drop NMU revision when doing a QA upload
-                my $upstream_version = $1;
-                my $debian_revision = $2;
-                $debian_revision++;
-                $start = "$upstream_version-$debian_revision";
-                $end = "";
+		# Drop NMU revision when doing a QA upload
+		my $upstream_version = $1;
+		my $debian_revision = $2;
+		$debian_revision++;
+		$start = "$upstream_version-$debian_revision";
+		$end = "";
 	    } elsif ($opt_bpo and not $start =~ /~bpo[0-9]+\+$/) {
 		# If it's not already a backport make it so
 		# otherwise we can be safe if we behave like dch -i
@@ -1058,7 +1084,14 @@ if (($opt_i || $opt_n || $opt_bn || $opt_R || $opt_qa || $opt_s || $opt_bpo || $
 	    } elsif (!$opt_news) {
 		# Don't bump the version of a NEWS file in this case as we're
 		# using the version from the changelog
-		$end++;
+
+		if (($opt_i or $opt_s) and $distributor eq 'Ubuntu' and
+		    $start !~ /(build|ubuntu|prop(osed)?|~ppa)(\d+\.)*$/ and
+		    not $opt_U) {
+		    $end .= "ubuntu1";
+		} else {
+		    $end++;
+		}
 
 		# Attempt to set the distribution for a backport correctly
 		# based on the version of the previous backport
@@ -1074,6 +1107,17 @@ if (($opt_i || $opt_n || $opt_bn || $opt_R || $opt_qa || $opt_s || $opt_bpo || $
 		    }
 		}
 
+		if(! ($opt_s or $opt_n or $distributor eq 'Ubuntu')) {
+		    if ($start =~/(.*?)-(\d+)\.$/) {
+			# Drop NMU revision
+			my $upstream_version = $1;
+			my $debian_revision = $2;
+			$debian_revision++;
+			$start = "$upstream_version-$debian_revision";
+			$end = "";
+		    }
+		}
+
 		if (! ($opt_qa or $opt_bpo or $opt_l)) {
 		    $useextra = 1;
 		}
@@ -1101,7 +1145,7 @@ if (($opt_i || $opt_n || $opt_bn || $opt_R || $opt_qa || $opt_s || $opt_bpo || $
 	$bpo_dist ||= $bpo_dists{$latest_bpo_dist} . '-backports';
     }
     my $distribution = $opt_D || $bpo_dist || (($opt_release_heuristic eq 'changelog') ? "UNRELEASED" : $DISTRIBUTION);
-    
+
     my $urgency = $opt_u;
     if ($opt_news) {
 	$urgency ||= $CL_URGENCY;
@@ -1117,34 +1161,37 @@ if (($opt_i || $opt_n || $opt_bn || $opt_R || $opt_qa || $opt_s || $opt_bpo || $
     } else {
 	print O "$PACKAGE ($NEW_VERSION) $distribution; urgency=$urgency\n\n";
 	if ($opt_n && ! $opt_news) {
-            print O "  * Non-maintainer upload.\n";
+	    print O "  * Non-maintainer upload.\n";
 	    $line = 1;
 	} elsif ($opt_bn && ! $opt_news) {
-            my $arch = qx/dpkg-architecture -qDEB_BUILD_ARCH/; chomp ($arch);
-            print O "  * Binary-only non-maintainer upload for $arch; no source changes.\n";
-            $line = 1;
+	    my $arch = qx/dpkg-architecture -qDEB_BUILD_ARCH/; chomp ($arch);
+	    print O "  * Binary-only non-maintainer upload for $arch; no source changes.\n";
+	    $line = 1;
 	} elsif ($opt_qa && ! $opt_news) {
-            print O "  * QA upload.\n";
-            $line = 1;
+	    print O "  * QA upload.\n";
+	    $line = 1;
 	} elsif ($opt_s && ! $opt_news) {
-            if ($distributor eq 'Ubuntu') {
-                print O "  * SECURITY UPDATE:\n";
-                print O "  * References\n";
-            } else {
-                print O "  * Non-maintainer upload by the Security Team.\n";
-            }
-            $line = 1;
+	    if ($distributor eq 'Ubuntu') {
+		print O "  * SECURITY UPDATE:\n";
+		print O "  * References\n";
+	    } else {
+		print O "  * Non-maintainer upload by the Security Team.\n";
+	    }
+	    $line = 1;
+	} elsif ($opt_team && ! $opt_news) {
+	    print O "  * Team upload.\n";
+	    $line = 1;
 	} elsif ($opt_bpo && ! $opt_news) {
-            print O "  * Rebuild for $bpo_dist.\n";
-            $line = 1;
+	    print O "  * Rebuild for $bpo_dist.\n";
+	    $line = 1;
 	}
 	if (@closes_text or $TEXT) {
-            foreach (@closes_text) { format_line($_, 1); }
-            if (length $TEXT) { format_line($TEXT, 1); }
+	    foreach (@closes_text) { format_line($_, 1); }
+	    if (length $TEXT) { format_line($TEXT, 1); }
 	} elsif ($opt_news) {
-            print O "  \n";
+	    print O "  \n";
 	} else {
-            print O "  * \n";
+	    print O "  * \n";
 	}
 	$line += 3;
 	print O "\n -- $MAINTAINER <$EMAIL>  $DATE\n\n";
@@ -1157,7 +1204,7 @@ if (($opt_i || $opt_n || $opt_bn || $opt_R || $opt_qa || $opt_s || $opt_bpo || $
 if (($opt_r || $opt_a || $merge) && ! $opt_create) {
     # This means we just have to generate a new * entry in changelog
     # and if a multi-developer changelog is detected, add developer names.
-    
+
     $NEW_VERSION=$VERSION unless $NEW_VERSION;
     $NEW_SVERSION=$SVERSION unless $NEW_SVERSION;
     $NEW_UVERSION=$UVERSION unless $NEW_UVERSION;
@@ -1181,19 +1228,26 @@ if (($opt_r || $opt_a || $merge) && ! $opt_create) {
 	    # maintainer
 	    $nextmaint ||= $1;
 	}
-	elsif (defined $lastmaint) {
-	    if (m/^\w[-+0-9a-z.]* \([^\(\) \t]+\)((?:\s+[-+0-9a-z.]+)+)\;\s+urgency=(\w+)/i) {
+	elsif (m/^\w[-+0-9a-z.]* \(([^\(\) \t]+)\)((?:\s+[-+0-9a-z.]+)+)\;\s+urgency=(\w+)/i) {
+	    if (defined $lastmaint) {
 		$lastheader = $_;
-		$lastdist = $1;
+		$lastdist = $2;
 		$lastdist =~ s/^\s+//;
 		undef $lastdist if $lastdist eq "UNRELEASED";
 		# Revert to our previously saved position
 		$line = $savedline;
 		last;
 	    }
-	}	
-	elsif (/  \* Upload to ([^ ]+).*$/) {
-	    ($dist_indicator = $1) =~ s/[,;]$//;
+	    else {
+		my $tmpver = $1;
+		$tmpver =~ s/^\s+//;
+		if ($tmpver =~ m/~bpo(\d+)\+/ && exists $bpo_dists{$1}) {
+		    $dist_indicator = "$bpo_dists{$1}-backports";
+		}
+	    }
+	}
+	elsif (/  \* (?:Upload to|Rebuild for) (\S+).*$/) {
+	    ($dist_indicator = $1) =~ s/[!:.,;]$//;
 	    chomp $dist_indicator;
 	}
 	elsif (/^ --\s+([^<]+)\s+/) {
@@ -1211,7 +1265,7 @@ if (($opt_r || $opt_a || $merge) && ! $opt_create) {
     my $multimaint=0;
     if (! $opt_news) {
 	my $lastmultimaint;
-	
+
 	# Parse the changelog for multi-maintainer maintainer lines of
 	# the form [ Full Name ] and record the last of these.
 	while ($CHANGES=~/.*\n^\s+\[\s+([^\]]+)\s+]\s*$/mg) {
@@ -1226,7 +1280,7 @@ if (($opt_r || $opt_a || $merge) && ! $opt_create) {
 	    (defined $nextmaint)
 	   ) {
 	    $multimaint=1;
-	
+
 	    if (! $lastmultimaint) {
 		# Add a multi-maintainer header to the top of the existing
 		# changelog.
@@ -1251,7 +1305,7 @@ if (($opt_r || $opt_a || $merge) && ! $opt_create) {
 		    $distribution = $dist_indicator;
 		} else {
 		    if ($distributor eq 'Ubuntu') {
-			$distribution = $opt_D || "karmic";
+			$distribution = $opt_D || "oneiric";
 		    } else {
 			$distribution = $opt_D || $lastdist || "unstable";
 		    }
@@ -1286,7 +1340,7 @@ if (($opt_r || $opt_a || $merge) && ! $opt_create) {
     };
 
     if (! $opt_r) {
-    	# Add a multi-maintainer header...
+	# Add a multi-maintainer header...
 	if ($multimaint) {
 	    # ...unless there already is one for this maintainer.
 	    if (!defined $maintline) {
@@ -1411,16 +1465,14 @@ if ((!$TEXT and !$EMPTY_TEXT and ! ($opt_create and $opt_empty)) or @closes_text
     system("sensible-editor +$line $changelog_path.dch") == 0 or
 	fatal "Error editing $changelog_path";
 
-    if (! @closes_text) { # so must have a changelog added by hand
-	my $newmtime = (stat("$changelog_path.dch"))[9];
-	defined $newmtime or fatal
-	    "Error getting modification time of temporary $changelog_path: $!";
-	if ($mtime == $newmtime && ! $opt_create &&
-	    (!$opt_r || ($opt_r && $opt_force_save_on_release))) {
+    my $newmtime = (stat("$changelog_path.dch"))[9];
+    defined $newmtime or fatal
+	"Error getting modification time of temporary $changelog_path: $!";
+    if ($mtime == $newmtime && ! $opt_create &&
+	(!$opt_r || ($opt_r && $opt_force_save_on_release))) {
 
-	    warn "$progname: $changelog_path unmodified; exiting.\n";
-	    exit 0;
-	}
+	warn "$progname: $changelog_path unmodified; exiting.\n";
+	exit 0;
     }
 }
 
@@ -1459,32 +1511,34 @@ if ((basename(cwd()) =~ m%^\Q$PACKAGE\E-\Q$UVERSION\E$%) &&
 	} else {
 	    warn "$progname warning: Couldn't rename directory: $!";
 	}
-       # And check whether a new orig tarball exists
-       my @origs = glob("../$PACKAGE\_$new_uversion.*");
-       my $num_origs = grep { /^..\/\Q$PACKAGE\E_\Q$new_uversion\E\.orig\.tar\.(gz|bz2|lzma)$/ } @origs;
-       if ($num_origs == 0) {
-          warn "$progname warning: no orig tarball found for the new version.\n";
-       }
+	# And check whether a new orig tarball exists
+	my @origs = glob("../$PACKAGE\_$new_uversion.*");
+	my $num_origs = grep { /^..\/\Q$PACKAGE\E_\Q$new_uversion\E\.orig\.tar\.(gz|bz2|lzma|xz)$/ } @origs;
+	if ($num_origs == 0) {
+	    warn "$progname warning: no orig tarball found for the new version.\n";
+	}
     }
 }
 
 exit 0;
 
-
-# Format for standard Debian changelogs
-format CHANGELOG =
+{
+    no warnings 'uninitialized';
+    # Format for standard Debian changelogs
+    format CHANGELOG =
   * ^<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
     $CHGLINE
  ~~ ^<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
     $CHGLINE
 .
-# Format for NEWS files.
-format NEWS =
+    # Format for NEWS files.
+    format NEWS =
   ^<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
     $CHGLINE
 ~~^<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
     $CHGLINE
 .
+}
 
 my $linecount=0;
 sub format_line {
@@ -1492,10 +1546,13 @@ sub format_line {
     my $newentry=shift;
 
     # Work around the fact that write() with formats
-    # seems to assume that character == byte
+    # seems to assume that characters are single-byte
     # See http://rt.perl.org/rt3/Public/Bug/Display.html?id=33832
-    # and Debian bug #473769
-    my $count = () = $CHGLINE =~ /[^\x00-\x7F]/mg;
+    # and Debian bugs #473769 and #541484
+    # This relies on $CHGLINE being a sequence of unicode characters.  We can
+    # compare how many unicode characters we have to how many bytes we have
+    # when encoding to utf8 and therefore how many spaces we need to pad.
+    my $count = length(encode_utf8($CHGLINE)) - length($CHGLINE);
     $CHGLINE .= " " x $count;
 
     print O "\n" if $opt_news && ! ($newentry || $linecount);
