@@ -82,6 +82,9 @@ Valid options are:
    --auto-ver-sort        When comparing source packages, ensure the
                           comparison is performed in version order
    --no-auto-ver-sort     Do not do so (default)
+   --unpack-tarballs      Unpack tarballs found in the top level source
+                          directory (default)
+   --no-unpack-tarballs   Do not do so
 
 Default settings modified by devscripts configuration files:
 $modified_conf_msg
@@ -109,6 +112,7 @@ my @diff_opts = ();
 my $show_diffstat = 0;
 my $wdiff_source_control = 0;
 my $auto_ver_sort = 0;
+my $unpack_tarballs = 1;
 
 my $quiet = 0;
 
@@ -129,6 +133,7 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
 		       'DEBDIFF_SHOW_DIFFSTAT' => 'no',
 		       'DEBDIFF_WDIFF_SOURCE_CONTROL' => 'no',
 		       'DEBDIFF_AUTO_VER_SORT' => 'no',
+		       'DEBDIFF_UNPACK_TARBALLS' => 'yes',
 		       );
     my %config_default = %config_vars;
 
@@ -157,6 +162,8 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
 	or $config_vars{'DEBDIFF_WDIFF_SOURCE_CONTROL'}='no';
     $config_vars{'DEBDIFF_AUTO_VER_SORT'} =~ /^(yes|no)$/
 	or $config_vars{'DEBDIFF_AUTO_VER_SORT'}='no';
+    $config_vars{'DEBDIFF_UNPACK_TARBALLS'} =~ /^(yes|no)$/
+	or $config_vars{'DEBDIFF_UNPACK_TARBALLS'}='yes';
 
     foreach my $var (sort keys %config_vars) {
 	if ($config_vars{$var} ne $config_default{$var}) {
@@ -175,6 +182,7 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
     $wdiff_source_control = $config_vars{'DEBDIFF_WDIFF_SOURCE_CONTROL'}
 	eq 'yes' ? 1 : 0;
     $auto_ver_sort = $config_vars{'DEBDIFF_AUTO_VER_SORT'} eq 'yes' ? 1 : 0;
+    $unpack_tarballs = $config_vars{'DEBDIFF_UNPACK_TARBALLS'} eq 'yes' ? 1 : 0;
 
 }
 
@@ -253,7 +261,7 @@ while (@ARGV) {
     elsif ($ARGV[0] eq '--from') { $type = 'debs'; last; }
     elsif ($ARGV[0] =~ /^--w([plt])$/) { $wdiff_opt = "-$1"; shift; }
     elsif ($ARGV[0] =~ /^(--ignore-space|-w)$/) {
-	push @diff_opts, "-w"; 
+	push @diff_opts, "-w";
 	shift;
     }
     elsif ($ARGV[0] eq '--diffstat') { $show_diffstat = 1; shift; }
@@ -262,6 +270,8 @@ while (@ARGV) {
     elsif ($ARGV[0] =~ /^--no-?wdiff-source-control$/) { $wdiff_source_control = 0; shift; }
     elsif ($ARGV[0] eq '--auto-ver-sort') { $auto_ver_sort = 1; shift; }
     elsif ($ARGV[0] =~ /^--no-?auto-ver-sort$/) { $auto_ver_sort = 0; shift; }
+    elsif ($ARGV[0] eq '--unpack-tarballs') { $unpack_tarballs = 1; shift; }
+    elsif ($ARGV[0] =~ /^--no-?unpack-tarballs$/) { $unpack_tarballs = 0; shift; }
     elsif ($ARGV[0] =~ /^--no-?conf$/) {
 	fatal "--no-conf is only acceptable as the first command-line option!";
     }
@@ -449,7 +459,7 @@ elsif ($type eq 'dsc') {
 		if ($file =~ /\.diff\.gz$/) {
 		    $diffs[$i] = cwd() . '/' . $file;
 		}
-		elsif ($file =~ /(\.orig)?\.tar\.(gz|bz2|lzma)$/) {
+		elsif ($file =~ /(\.orig)?\.tar\.(gz|bz2|lzma|xz)$/) {
 		    $origs[$i] = $file;
 		}
 	    } else {
@@ -553,6 +563,9 @@ elsif ($type eq 'dsc') {
 		    last;
 	    }
 	    closedir(DIR);
+
+	    # also unpack tarballs found in the top level source directory so we can compare their contents too
+	    next unless $unpack_tarballs;
 	    opendir DIR,${"dir$i"}.'/'.${"sdir$i"};
 
 	    my $tarballs = 1;
@@ -562,7 +575,7 @@ elsif ($type eq 'dsc') {
 		    if ($_ =~ /tar.gz$/) {
 			$filename =~ s%(.*)\.tar\.gz$%$1%;
 			$tarballs++;
-		        system qq(cd ${"dir$i"}/${"sdir$i"} && tar zxf $_ >/dev/null && test -d $filename && mv $filename $unpacked); 
+		        system qq(cd ${"dir$i"}/${"sdir$i"} && tar zxf $_ >/dev/null && test -d $filename && mv $filename $unpacked);
 		    }
 		    if ($_ =~ /tar.bz$/ || $_ =~ /tar.bz2$/) {
 			$filename =~ s%(.*)\.tar\.bz2?$%$1%;
@@ -679,7 +692,7 @@ if ($show_moved and $type ne 'deb') {
 	print join("\n",@deblosses), "\n\n";
     }
 
-    # We start by determining which files are in the first set of debs, the 
+    # We start by determining which files are in the first set of debs, the
     # second set of debs or both.
     my %files;
     grep $files{$_}--, @D1;
@@ -721,7 +734,7 @@ if ($show_moved and $type ne 'deb') {
 	# list it separately.
 	my @fdebs1 = split (/:/, $files1{$file});
 	my @fdebs2 = split (/:/, $files2{$file});
-	
+
 	if (@fdebs1 == 1 && @fdebs2 == 1) {
 	    push @{$changes{$fdebs1[0]}{$fdebs2[0]}}, $file;
 	} else {
