@@ -1,4 +1,4 @@
-#! /bin/bash
+#! /bin/sh
 
 # This program is designed to PGP sign a .dsc and .changes file pair
 # in the form needed for a legal Debian upload.  It is based in part
@@ -31,8 +31,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 # Abort if anything goes wrong
 set -e
@@ -45,7 +44,7 @@ MODIFIED_CONF_MSG='Default settings modified by devscripts configuration files:'
 signingdir=""
 remotefilesdir=""
 
-trap "cleanup_tmpdir" 0 1 2 3 9 11 13 15
+trap "cleanup_tmpdir" EXIT HUP INT QUIT KILL SEGV PIPE TERM
 
 # --- Functions
 
@@ -88,6 +87,8 @@ usage () {
     -a<arch>        Use changes file made for Debian target architecture <arch>
     -t<target>      Use changes file made for GNU target architecture <target>
     --multi         Use most recent multiarch .changes file found
+    --re-sign       Re-sign if the file is already signed.
+    --no-re-sign    Don't re-sign if the file is already signed.
     --debs-dir <directory>
                     The location of the .changes / .dsc files when called from
                     within a source tree (default "..")
@@ -224,9 +225,14 @@ withecho () {
 # and failure if the file needs signing.  Parameters: $1=filename,
 # $2=file description for message (dsc or changes)
 check_already_signed () {
-    if [ "`head -n 1 \"$1\"`" != "-----BEGIN PGP SIGNED MESSAGE-----" ]
-    then
+    [ "`head -n 1 \"$1\"`" = "-----BEGIN PGP SIGNED MESSAGE-----" ] || \
 	return 1
+
+    local resign
+    if [ "$opt_re_sign" = "true" ]; then
+	resign="true"
+    elif [ "$opt_re_sign" = "false" ]; then
+	resign="false"
     else
 	response=n
 	if [ -z "$DEBSIGN_ALWAYS_RESIGN" ]; then
@@ -234,16 +240,19 @@ check_already_signed () {
 	    read response
 	fi
 	case $response in
-	[Nn]*)
-	    UNSIGNED_FILE="$(temp_filename "$1" "unsigned")"
-
-	    sed -e '1,/^$/d; /^$/,$d' "$1" > "$UNSIGNED_FILE"
-	    movefile "$UNSIGNED_FILE" "$1"
-	    return 1
-	    ;;
-	*) return 0;;
+	[Nn]*) resign="true" ;;
+	*)     resign="false" ;;
 	esac
     fi
+
+    [ "$resign" = "true" ] || \
+	return 0
+
+    UNSIGNED_FILE="$(temp_filename "$1" "unsigned")"
+
+    sed -e '1,/^$/d; /^$/,$d' "$1" > "$UNSIGNED_FILE"
+    movefile "$UNSIGNED_FILE" "$1"
+    return 1
 }
 
 # --- main script
@@ -252,14 +261,14 @@ check_already_signed () {
 unset GREP_OPTIONS
 
 # Boilerplate: set config variables
+DEFAULT_DEBSIGN_ALWAYS_RESIGN=
 DEFAULT_DEBSIGN_PROGRAM=
 DEFAULT_DEBSIGN_SIGNLIKE=
 DEFAULT_DEBSIGN_MAINT=
 DEFAULT_DEBSIGN_KEYID=
-DEFAULT_DEBSIGN_ALWAYS_RESIGN=
 DEFAULT_DEBRELEASE_DEBS_DIR=..
-VARS="DEBSIGN_PROGRAM DEBSIGN_SIGNLIKE DEBSIGN_MAINT DEBSIGN_KEYID DEBSIGN_ALWAYS_RESIGN"
-VARS="$VARS DEBRELEASE_DEBS_DIR"
+VARS="DEBSIGN_ALWAYS_RESIGN DEBSIGN_PROGRAM DEBSIGN_SIGNLIKE DEBSIGN_MAINT"
+VARS="$VARS DEBSIGN_KEYID DEBRELEASE_DEBS_DIR"
 
 if [ "$1" = "--no-conf" -o "$1" = "--noconf" ]; then
     shift
@@ -349,6 +358,8 @@ do
 	-a*)	targetarch="$value" ;;
 	-t*)	targetgnusystem="$value" ;;
 	--multi) multiarch="true" ;;
+	--re-sign)    opt_re_sign="true" ;;
+	--no-re-sign) opt_re_sign="false" ;;
 	-r*)	if [ -n "$value" ]; then remotehost=$value;
 		elif [ $# -lt 1 ]; then
 		    echo "$PROGNAME: -r option missing argument!" >&2
@@ -574,7 +585,7 @@ dosigning() {
 	}
 
 	# simple validator for .commands files, see
-	# ftp://ftp-master.debian.org/pub/UploadQueue/README
+	# ftp://ftp.upload.debian.org/pub/UploadQueue/README
 	perl -ne 'BEGIN { $uploader = 0; $incommands = 0; }
               END { exit $? if $?;
                     if ($uploader && $incommands) { exit 0; }
@@ -602,7 +613,7 @@ dosigning() {
                  if (/./) { die ".commands file: extra stuff after Commands field!\n"; }
               }' $commands || {
 	echo "$PROGNAME: .commands file appears to be invalid. see:
-ftp://ftp-master.debian.org/pub/UploadQueue/README
+ftp://ftp.upload.debian.org/pub/UploadQueue/README
 for valid format" >&2;
 	exit 1; }
 
