@@ -19,12 +19,12 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 use 5.008;  # uses 'our' variables and filetest
 use strict;
 use Cwd;
+use Cwd 'abs_path';
 use File::Basename;
 use File::Copy;
 use File::Temp qw/tempdir/;
@@ -108,7 +108,7 @@ Options:
                    What constitutes a matching directory name; REGEX is
                    a Perl regular expression; the string \`PACKAGE\' will
                    be replaced by the package name; see manpage for details
-                   (default: 'PACKAGE(-.*)?')
+                   (default: 'PACKAGE(-.+)?')
     --watchfile FILE
                    Specify the watchfile rather than using debian/watch;
                    no directory traversing will be done in this case
@@ -119,6 +119,8 @@ Options:
                    Specify the version which the upstream release must
                    match in order to be considered, rather than using the
                    release with the highest version
+    --download-current-version
+                   Download the currently packaged version
     --package PACKAGE
                    Specify the package name rather than examining
                    debian/changelog; must use --upstream-version and
@@ -166,7 +168,7 @@ my $repack = 0; # repack .tar.bz2 or .zip to .tar.gz
 my $symlink = 'symlink';
 my $verbose = 0;
 my $check_dirname_level = 1;
-my $check_dirname_regex = 'PACKAGE(-.*)?';
+my $check_dirname_regex = 'PACKAGE(-.+)?';
 my $dehs = 0;
 my %dehs_tags;
 my $dehs_end_output = 0;
@@ -191,7 +193,7 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
 		       'USCAN_USER_AGENT' => '',
 		       'USCAN_REPACK' => 'no',
 		       'DEVSCRIPTS_CHECK_DIRNAME_LEVEL' => 1,
-		       'DEVSCRIPTS_CHECK_DIRNAME_REGEX' => 'PACKAGE(-.*)?',
+		       'DEVSCRIPTS_CHECK_DIRNAME_REGEX' => 'PACKAGE(-.+)?',
 		       );
     my %config_default = %config_vars;
 
@@ -258,10 +260,11 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
 my $debug = 0;
 my ($opt_h, $opt_v, $opt_destdir, $opt_download, $opt_force_download,
     $opt_report, $opt_passive, $opt_symlink, $opt_repack);
-my ($opt_verbose, $opt_ignore, $opt_level, $opt_regex, $opt_noconf);
+my ($opt_verbose, $opt_level, $opt_regex, $opt_noconf);
 my ($opt_package, $opt_uversion, $opt_watchfile, $opt_dehs, $opt_timeout);
 my $opt_download_version;
 my $opt_user_agent;
+my $opt_download_current_version;
 
 GetOptions("help" => \$opt_h,
 	   "version" => \$opt_v,
@@ -282,13 +285,13 @@ GetOptions("help" => \$opt_h,
 	   "dehs!" => \$opt_dehs,
 	   "verbose!" => \$opt_verbose,
 	   "debug" => \$debug,
-	   "ignore-dirname" => \$opt_ignore,
 	   "check-dirname-level=s" => \$opt_level,
 	   "check-dirname-regex=s" => \$opt_regex,
 	   "user-agent=s" => \$opt_user_agent,
 	   "useragent=s" => \$opt_user_agent,
 	   "noconf" => \$opt_noconf,
 	   "no-conf" => \$opt_noconf,
+	   "download-current-version" => \$opt_download_current_version,
 	   )
     or die "Usage: $progname [options] [directories]\nRun $progname --help for more details\n";
 
@@ -318,11 +321,6 @@ if ($dehs) {
     $SIG{'__DIE__'} = \&dehs_die;
 }
 
-# dirname stuff
-if ($opt_ignore) {
-    die "$progname: --ignore-dirname has been replaced by --check-dirname-level and\n--check-dirname-regex; run $progname --help for more details\n";
-}
-
 if (defined $opt_level) {
     if ($opt_level =~ /^[012]$/) { $check_dirname_level = $opt_level; }
     else {
@@ -421,7 +419,7 @@ if (defined $opt_watchfile) {
 	if (! defined $package || ! defined $debversion) {
 	    die "$progname: Problems determining package name and/or version from\n  debian/changelog\n";
 	}
-	
+
 	# Check the directory is properly named for safety
 	my $good_dirname = 1;
 	if ($check_dirname_level ==  2 or
@@ -523,7 +521,7 @@ for my $dir (@dirs) {
 	    print "-- Found watchfile in $dir/debian\n" if $verbose;
 	} else {
 	    print "-- Skip watchfile in $dir/debian since it does not match the package name\n" .
-	        "   (or the settings of the--check-dirname-level and --check-dirname-regex options if any).\n"
+	        "   (or the settings of the --check-dirname-level and --check-dirname-regex options if any).\n"
 	        if $verbose;
 	    next;
 	}
@@ -558,7 +556,7 @@ if (defined $opt_uversion) {
 
 # Now sort the list of directories, so that we process the most recent
 # directories first, as determined by the package version numbers
-@debdirs = Devscripts::Versort::versort(@debdirs);
+@debdirs = Devscripts::Versort::deb_versort(@debdirs);
 
 # Now process the watchfiles in order.  If a directory d has subdirectories
 # d/sd1/debian and d/sd2/debian, which each contain watchfiles corresponding
@@ -614,12 +612,12 @@ exit ($found ? 0 : 1);
 # watch_version=2:
 #
 # For ftp sites:
-#   ftp://site.name/dir/path/pattern-(.*)\.tar\.gz [version [action]]
+#   ftp://site.name/dir/path/pattern-(.+)\.tar\.gz [version [action]]
 #
 # For http sites:
-#   http://site.name/dir/path/pattern-(.*)\.tar\.gz [version [action]]
+#   http://site.name/dir/path/pattern-(.+)\.tar\.gz [version [action]]
 # or
-#   http://site.name/dir/path/base pattern-(.*)\.tar\.gz [version [action]]
+#   http://site.name/dir/path/base pattern-(.+)\.tar\.gz [version [action]]
 #
 # Lines can be prefixed with opts=<opts>.
 #
@@ -630,51 +628,51 @@ exit ($found ? 0 : 1);
 # watch_version=3:
 #
 # Correct handling of regex special characters in the path part:
-# ftp://ftp.worldforge.org/pub/worldforge/libs/Atlas-C++/transitional/Atlas-C\+\+-(.*)\.tar\.gz
-# 
+# ftp://ftp.worldforge.org/pub/worldforge/libs/Atlas-C++/transitional/Atlas-C\+\+-(.+)\.tar\.gz
+#
 # Directory pattern matching:
 # ftp://ftp.nessus.org/pub/nessus/nessus-([\d\.]+)/src/nessus-core-([\d\.]+)\.tar\.gz
-# 
+#
 # The pattern in each part may contain several (...) groups and
 # the version number is determined by joining all groups together
 # using "." as separator.  For example:
 #   ftp://site/dir/path/pattern-(\d+)_(\d+)_(\d+)\.tar\.gz
-# 
+#
 # This is another way of handling site with funny version numbers,
 # this time using mangling.  (Note that multiple groups will be
 # concatenated before mangling is performed, and that mangling will
 # only be performed on the basename version number, not any path version
 # numbers.)
 # opts=uversionmangle=s/^/0.0./ \
-#   ftp://ftp.ibiblio.org/pub/Linux/ALPHA/wine/development/Wine-(.*)\.tar\.gz
-# 
+#   ftp://ftp.ibiblio.org/pub/Linux/ALPHA/wine/development/Wine-(.+)\.tar\.gz
+#
 # Similarly, the upstream part of the Debian version number can be
 # mangled:
 # opts=dversionmangle=s/\.dfsg\.\d+$// \
-#   http://some.site.org/some/path/foobar-(.*)\.tar\.gz
-# 
+#   http://some.site.org/some/path/foobar-(.+)\.tar\.gz
+#
 # The versionmangle=... option is a shorthand for saying uversionmangle=...
 # and dversionmangle=... and applies to both upstream and Debian versions.
-# 
+#
 # The option filenamemangle can be used to mangle the name under which
 # the downloaded file will be saved:
 #   href="http://foo.bar.org/download/?path=&amp;download=foo-0.1.1.tar.gz"
 # could be handled as:
 # opts=filenamemangle=s/.*=(.*)/$1/ \
-#     http://foo.bar.org/download/\?path=&amp;download=foo-(.*)\.tar\.gz
+#     http://foo.bar.org/download/\?path=&amp;download=foo-(.+)\.tar\.gz
 # and
 #   href="http://foo.bar.org/download/?path=&amp;download_version=0.1.1"
 # as:
 # opts=filenamemangle=s/.*=(.*)/foo-$1\.tar\.gz/ \
-#    http://foo.bar.org/download/\?path=&amp;download_version=(.*)
-# 
+#    http://foo.bar.org/download/\?path=&amp;download_version=(.+)
+#
 # The option downloadurlmangle can be used to mangle the URL of the file
 # to download.  This can only be used with http:// URLs.  This may be
 # necessary if the link given on the webpage needs to be transformed in
 # some way into one which will work automatically, for example:
 # opts=downloadurlmangle=s/prdownload/download/ \
 #   http://developer.berlios.de/project/showfiles.php?group_id=2051 \
-#   http://prdownload.berlios.de/softdevice/vdr-softdevice-(.*).tgz
+#   http://prdownload.berlios.de/softdevice/vdr-softdevice-(.+).tgz
 
 
 sub process_watchline ($$$$$$)
@@ -699,7 +697,7 @@ sub process_watchline ($$$$$$)
     %dehs_tags = ('package' => $pkg);
 
     if ($watch_version == 1) {
-	($site, $dir, $pattern, $lastversion, $action) = split ' ', $line, 5;
+	($site, $dir, $filepattern, $lastversion, $action) = split ' ', $line, 5;
 
 	if (! defined $lastversion or $site =~ /\(.*\)/ or $dir =~ /\(.*\)/) {
 	    warn "$progname warning: there appears to be a version 2 format line in\n  the version 1 watchfile $watchfile;\n  Have you forgotten a 'version=2' line at the start, perhaps?\n  Skipping the line: $line\n";
@@ -707,15 +705,15 @@ sub process_watchline ($$$$$$)
 	}
 	if ($site !~ m%\w+://%) {
 	    $site = "ftp://$site";
-	    if ($pattern !~ /\(.*\)/) {
+	    if ($filepattern !~ /\(.*\)/) {
 		# watch_version=1 and old style watchfile;
 		# pattern uses ? and * shell wildcards; everything from the
 		# first to last of these metachars is the pattern to match on
-		$pattern =~ s/(\?|\*)/($1/;
-		$pattern =~ s/(\?|\*)([^\?\*]*)$/$1)$2/;
-		$pattern =~ s/\./\\./g;
-		$pattern =~ s/\?/./g;
-		$pattern =~ s/\*/.*/g;
+		$filepattern =~ s/(\?|\*)/($1/;
+		$filepattern =~ s/(\?|\*)([^\?\*]*)$/$1)$2/;
+		$filepattern =~ s/\./\\./g;
+		$filepattern =~ s/\?/./g;
+		$filepattern =~ s/\*/.*/g;
 		$style='old';
 		warn "$progname warning: Using very old style of filename pattern in $watchfile\n  (this might lead to incorrect results): $3\n";
 	    }
@@ -726,6 +724,7 @@ sub process_watchline ($$$$$$)
 	$base =~ s%(?<!:)//%/%g;
 	$base =~ m%^(\w+://[^/]+)%;
 	$site = $1;
+	$pattern = $filepattern;
     } else {
 	# version 2/3 watchfile
 	if ($line =~ s/^opt(?:ion)?s=//) {
@@ -786,7 +785,7 @@ sub process_watchline ($$$$$$)
 	}
 
 	# Check all's OK
-	if ($filepattern !~ /\(.*\)/) {
+	if (not $filepattern or $filepattern !~ /\(.*\)/) {
 	    warn "$progname warning: Filename pattern missing version delimiters ()\n  in $watchfile, skipping:\n  $line\n";
 	    return 1;
 	}
@@ -820,6 +819,32 @@ sub process_watchline ($$$$$$)
 	$pattern = "(?:(?:$site)?" . quotemeta($basedir) . ")?$filepattern";
     }
 
+    if (! $lastversion or $lastversion eq 'debian') {
+	if (defined $pkg_version) {
+	    $lastversion=$pkg_version;
+	} else {
+	    warn "$progname warning: Unable to determine current version\n  in $watchfile, skipping:\n  $line\n";
+	    return 1;
+	}
+    }
+    # And mangle it if requested
+    my $mangled_lastversion;
+    $mangled_lastversion = $lastversion;
+    foreach my $pat (@{$options{'dversionmangle'}}) {
+	if (! safe_replace(\$mangled_lastversion, $pat)) {
+	    warn "$progname: In $watchfile, potentially"
+	      . " unsafe or malformed dversionmangle"
+	      . " pattern:\n  '$pat'"
+	      . " found. Skipping watchline\n"
+	      . "  $line\n";
+	    return 1;
+	}
+    }
+    if($opt_download_current_version) {
+	$download_version = $mangled_lastversion;
+	$force_download = 1;
+    }
+
     # Check all's OK
     if ($pattern !~ /\(.*\)/) {
 	warn "$progname warning: Filename pattern missing version delimiters ()\n  in $watchfile, skipping:\n  $line\n";
@@ -846,13 +871,13 @@ sub process_watchline ($$$$$$)
 	}
 
 	@redirections = @{$user_agent->get_redirections};
-	
+
 	print STDERR "$progname debug: redirections: @redirections\n"
 	    if $debug;
 
 	foreach my $_redir (@redirections) {
 	    my $base_dir = $_redir;
-	    
+
 	    $base_dir =~ s%^\w+://[^/]+/%/%;
 	    if ($_redir =~ m%^(\w+://[^/]+)%) {
 		my $base_site = $1;
@@ -912,7 +937,7 @@ sub process_watchline ($$$$$$)
 			foreach my $pat (@{$options{'uversionmangle'}}) {
 			    if (! safe_replace(\$mangled_version, $pat)) {
 				warn "$progname: In $watchfile, potentially"
-				  . " unsafe or malformed uversionmangle"
+			 	 . " unsafe or malformed uversionmangle"
 				  . " pattern:\n  '$pat'"
 				  . " found. Skipping watchline\n"
 				  . "  $line\n";
@@ -983,7 +1008,7 @@ sub process_watchline ($$$$$$)
 	# We separate out HTMLised listings from standard listings, so
 	# that we can target our search correctly
 	if ($content =~ /<\s*a\s+[^>]*href/i) {
-	    while ($content =~ 
+	    while ($content =~
 		m/(?:<\s*a\s+[^>]*href\s*=\s*\")((?-i)$pattern)\"/gi) {
 		my $file = $1;
 		my $mangled_version = join(".", $file =~ m/^$pattern$/);
@@ -1002,22 +1027,24 @@ sub process_watchline ($$$$$$)
 	} else {
 	    # they all look like:
 	    # info info ... info filename [ -> linkname]
-	    while ($content =~ m/\s($filepattern)(\s+->\s+\S+)?$/mg) {
-		my $file = $1;
-		my $mangled_version = join(".", $file =~ m/^$filepattern$/);
-		foreach my $pat (@{$options{'uversionmangle'}}) {
-		    if (! safe_replace(\$mangled_version, $pat)) {
-			warn "$progname: In $watchfile, potentially"
-			  . " unsafe or malformed uversionmangle"
-			  . " pattern:\n  '$pat'"
-			  . " found. Skipping watchline\n"
-			  . "  $line\n";
-			return 1;
+	    for my $ln (split(/\n/, $content)) {
+		if ($ln =~ m/\s($filepattern)(\s+->\s+\S+)?$/) {
+		    my $file = $1;
+		    my $mangled_version = join(".", $file =~ m/^$filepattern$/);
+		    foreach my $pat (@{$options{'uversionmangle'}}) {
+			if (! safe_replace(\$mangled_version, $pat)) {
+			    warn "$progname: In $watchfile, potentially"
+			      . " unsafe or malformed uversionmangle"
+			      . " pattern:\n  '$pat'"
+			      . " found. Skipping watchline\n"
+			      . "  $line\n";
+			    return 1;
+			}
 		    }
+		    push @files, [$mangled_version, $file];
 		}
-		push @files, [$mangled_version, $file];
 	    }
-	}	    
+	}
 
 	if (@files) {
 	    if ($verbose) {
@@ -1086,26 +1113,6 @@ EOF
 	    $newfile_base = "$pkg-$newversion.download";
 	}
     }
-    if (! $lastversion or $lastversion eq 'debian') {
-	if (defined $pkg_version) {
-	    $lastversion=$pkg_version;
-	} else {
-	    warn "$progname warning: Unable to determine current version\n  in $watchfile, skipping:\n  $line\n";
-	    return 1;
-	}
-    }
-    # And mangle it if requested
-    my $mangled_lastversion = $lastversion;
-    foreach my $pat (@{$options{'dversionmangle'}}) {
-	if (! safe_replace(\$mangled_lastversion, $pat)) {
-	    warn "$progname: In $watchfile, potentially"
-	      . " unsafe or malformed dversionmangle"
-	      . " pattern:\n  '$pat'"
-	      . " found. Skipping watchline\n"
-	      . "  $line\n";
-	    return 1;
-	}
-    }
 
     # So what have we got to report now?
     my $upstream_url;
@@ -1115,6 +1122,11 @@ EOF
 	if ($newfile =~ m%^\w+://%) {
 	    $upstream_url = $newfile;
 	}
+	elsif ($newfile =~ m%^//%) {
+	    $upstream_url = $site;
+	    $upstream_url =~ s/^(https?:).*/$1/;
+	    $upstream_url .= $newfile;
+	}
 	# absolute filename?
 	elsif ($newfile =~ m%^/%) {
 	    # Were there any redirections? If so try using those first
@@ -1190,7 +1202,7 @@ EOF
 
     # Can't just use $lastversion eq $newversion, as then 0.01 and 0.1
     # compare different, whereas they are treated as equal by dpkg
-    if (system("dpkg", "--compare-versions", "'$mangled_lastversion'", "eq", "'$newversion'") == 0) {
+    if (system("dpkg", "--compare-versions", "$mangled_lastversion", "eq", "$newversion") == 0) {
 	if ($verbose or ($download == 0 and $report and ! $dehs)) {
 	    print $pkg_report_header;
 	    $pkg_report_header = '';
@@ -1217,7 +1229,7 @@ EOF
     # We use dpkg's rules to determine whether our current version
     # is newer or older than the remote version.
     if (!defined $download_version) {
-        if (system("dpkg", "--compare-versions", "'$mangled_lastversion'", "gt", "'$newversion'") == 0) {
+	if (system("dpkg", "--compare-versions", "$mangled_lastversion", "gt", "$newversion") == 0) {
 	    if ($verbose) {
 		print " => remote site does not even have current version\n";
 	    } elsif ($dehs) {
@@ -1340,6 +1352,24 @@ EOF
 	$newfile_base = $newfile_base_gz;
     }
 
+    if ($repack and $newfile_base =~ /^(.*)\.(tar\.lzma|tlz(?:ma?)?)$/) {
+	print "-- Repacking from lzma to gzip\n" if $verbose;
+	my $newfile_base_gz = "$1.tar.gz";
+	system("xz -F lzma -cd $destdir/$newfile_base | gzip -n -9 > $destdir/$newfile_base_gz") == 0
+	  or die "repacking from lzma to gzip failed\n";
+	unlink "$destdir/$newfile_base";
+	$newfile_base = $newfile_base_gz;
+    }
+
+    if ($repack and $newfile_base =~ /^(.*)\.(tar\.xz|txz)$/) {
+	print "-- Repacking from xz to gzip\n" if $verbose;
+	my $newfile_base_gz = "$1.tar.gz";
+	system("xz -cd $destdir/$newfile_base | gzip -n -9 > $destdir/$newfile_base_gz") == 0
+	  or die "repacking from xz to gzip failed\n";
+	unlink "$destdir/$newfile_base";
+	$newfile_base = $newfile_base_gz;
+    }
+
     if ($repack and $newfile_base =~ /^(.*)\.zip$/) {
 	print "-- Repacking from zip to .tar.gz\n" if $verbose;
 
@@ -1348,15 +1378,22 @@ EOF
 
 	my $newfile_base_gz = "$1.tar.gz";
 	my $tempdir = tempdir ( "uscanXXXX", TMPDIR => 1, CLEANUP => 1 );
-	system("unzip -q -d $tempdir $destdir/$newfile_base; GZIP=-9 tar -C $tempdir -czf $destdir/$newfile_base_gz .") == 0 
-	  or die("Repacking from zip to tar.gz failed\n");
+	my $globpattern = "*";
+	my $hidden = ".[!.]*";
+	my $absdestdir = abs_path($destdir);
+	system("unzip -q -a -d $tempdir $destdir/$newfile_base") == 0
+	  or die("Repacking from zip to tar.gz failed (could not unzip)\n");
+	if (defined glob("$tempdir/$hidden")) {
+	    $globpattern .= " $hidden";
+	}
+	system("cd $tempdir; GZIP=-9 tar --owner=root --group=root --mode=a+rX -czf $absdestdir/$newfile_base_gz $globpattern") == 0
+	  or die("Repacking from zip to tar.gz failed (could not create tarball)\n");
 	unlink "$destdir/$newfile_base";
 	$newfile_base = $newfile_base_gz;
     }
 
     if ($newfile_base =~ /\.(tar\.gz|tgz|tar\.bz2|tbz2?)$/) {
-	my $filetype = `file $destdir/$newfile_base`;
-	$filetype =~ s%^\.\./\Q$newfile_base\E: %%;
+	my $filetype = `file -b -k $destdir/$newfile_base`;
 	unless ($filetype =~ /compressed data/) {
 	    warn "$progname warning: $destdir/$newfile_base does not appear to be a compressed file;\nthe file command says: $filetype\nNot processing this file any further!\n";
 	    return 1;
@@ -1397,17 +1434,23 @@ EOF
     } elsif ($dehs) {
 	my $msg = "Successfully downloaded updated package $newfile_base";
 	if ($newfile_base =~ /\.(tar\.gz|tgz)$/) {
+	    $dehs_tags{'target'} = "${pkg}_${newversion}.orig.tar.gz";
 	    if ($symlink eq 'symlink') {
 		$msg .= " and symlinked ${pkg}_${newversion}.orig.tar.gz to it";
 	    } elsif ($symlink eq 'rename') {
 		$msg .= " and renamed it as ${pkg}_${newversion}.orig.tar.gz";
+	    } else {
+		$dehs_tags{'target'} = $newfile_base;
 	    }
 	}
 	elsif ($newfile_base =~ /\.(tar\.bz2|tbz2?)$/) {
+	    $dehs_tags{'target'} = "${pkg}_${newversion}.orig.tar.bz2";
 	    if ($symlink eq 'symlink') {
 		$msg .= " and symlinked ${pkg}_${newversion}.orig.tar.bz2 to it";
 	    } elsif ($symlink eq 'rename') {
 		$msg .= " and renamed it as ${pkg}_${newversion}.orig.tar.bz2";
+	    } else {
+		$dehs_tags{'target'} = $newfile_base;
 	    }
 	}
 	dehs_msg($msg);
@@ -1472,7 +1515,10 @@ sub recursive_regex_dir ($$$) {
 
     $base =~ m%^(\w+://[^/]+)/(.*)$%;
     my $site = $1;
-    my @dirs = split /(\/)/, $2;
+    my @dirs = ();
+    if (defined $2) {
+	@dirs = split /(\/)/, $2;
+    }
     my $dir = '/';
 
     foreach my $dirpattern (@dirs) {
@@ -1533,7 +1579,7 @@ sub newest_dir ($$$$$) {
 	while ($content =~ m/<\s*a\s+[^>]*href\s*=\s*([\"\'])(.*?)\1/gi) {
 	    my $href = $2;
 	    if ($href =~ m&^$dirpattern/?$&) {
-		my $mangled_version = join(".", $href =~ m&^$dirpattern/?$&);
+		my $mangled_version = join(".", map { $_ || '' } $href =~ m&^$dirpattern/?$&);
 		push @hrefs, [$mangled_version, $href];
 	    }
 	}
@@ -1588,7 +1634,7 @@ sub newest_dir ($$$$$) {
 	# We separate out HTMLised listings from standard listings, so
 	# that we can target our search correctly
 	if ($content =~ /<\s*a\s+[^>]*href/i) {
-	    while ($content =~ 
+	    while ($content =~
 		m/(?:<\s*a\s+[^>]*href\s*=\s*\")((?-i)$pattern)\"/gi) {
 		my $dir = $1;
 		my $mangled_version = join(".", $dir =~ m/^$pattern$/);
@@ -1597,12 +1643,14 @@ sub newest_dir ($$$$$) {
 	} else {
 	    # they all look like:
 	    # info info ... info filename [ -> linkname]
-	    while ($content =~ m/($pattern)(\s+->\s+\S+)?$/mg) {
-		my $dir = $1;
-		my $mangled_version = join(".", $dir =~ m/^$pattern$/);
-		push @dirs, [$mangled_version, $dir];
+	    foreach my $ln (split(/\n/, $content)) {
+		if ($ln =~ m/($pattern)(\s+->\s+\S+)?$/) {
+		    my $dir = $1;
+		    my $mangled_version = join(".", $dir =~ m/^$pattern$/);
+		    push @dirs, [$mangled_version, $dir];
+		}
 	    }
-	}	    
+	}
 	if (@dirs) {
 	    if ($debug) {
 		print STDERR "-- Found the following matching dirs:\n";
@@ -1674,7 +1722,7 @@ sub process_watchfile ($$$$)
 	} elsif ($download == 0 and ! $dehs) {
 	    $pkg_report_header = "Processing watchfile line for package $package...\n";
 	}
-	    
+
 	$status +=
 	    process_watchline($_, $watch_version, $dir, $package, $version,
 			      $watchfile);
@@ -1724,7 +1772,7 @@ sub dehs_output ()
 
     for my $tag (qw(package debian-uversion debian-mangled-uversion
 		    upstream-version upstream-url
-		    status messages warnings errors)) {
+		    status target messages warnings errors)) {
 	if (exists $dehs_tags{$tag}) {
 	    if (ref $dehs_tags{$tag} eq "ARRAY") {
 		foreach my $entry (@{$dehs_tags{$tag}}) {
@@ -1840,7 +1888,7 @@ sub safe_replace($$) {
     if ($op eq 'tr' or $op eq 'y') {
 	$safeflags =~ tr/cds//cd;
 	return 0 if $safeflags ne $flags;
-	
+
 	$regexp =~ s/\\(.)/$1/g;
 	$replacement =~ s/\\(.)/$1/g;
 
@@ -1944,7 +1992,7 @@ sub safe_replace($$) {
 	    # Actually do the replacement
 	    substr $$in, $first, $last - $first, $replacement;
 	    # Update position
-	    $pos += length($replacement)- ($last - $first);
+	    $pos += length($replacement) - ($last - $first);
 
 	    if ($global) {
 		$replacement = $orig_replacement;
