@@ -17,8 +17,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 
 # Command line syntax is one of:
@@ -48,7 +47,7 @@ usage () {
 "Usage for a new archive:
   $PROGNAME [options] <new upstream archive> [<version>]
 For a patch file:
-  $PROGNAME [options] --patch|-p <patch>[.gz|.bz2|.lzma]
+  $PROGNAME [options] --patch|-p <patch>[.gz|.bz2|.lzma|.xz]
 Options are:
    --upstream-version <version>, -v <version>
                       specify version number of upstream package
@@ -56,10 +55,10 @@ Options are:
                       which command to be used to become root
                       for package-building
    --pristine, -u     Source is pristine upstream source and should be
-                      copied to <pkg>_<version>.orig.tar.{gz|bz2|lzma}; not valid
+                      copied to <pkg>_<version>.orig.tar.{gz|bz2|lzma|xz}; not valid
                       for patches
    --no-symlink       Copy new upstream archive to new location
-                      as <pkg>_<version>.orig.tar.{gz|bz2|lzma} instead of making a
+                      as <pkg>_<version>.orig.tar.{gz|bz2|lzma|xz} instead of making a
                       symlink
    --no-conf, --noconf
                       Don't read devscripts config files;
@@ -108,11 +107,13 @@ DEFAULT_UUPDATE_SYMLINK_ORIG=yes
 VARS="UUPDATE_ROOTCMD UUPDATE_PRISTINE UUPDATE_SYMLINK_ORIG"
 SUFFIX="1"
 
-case "$(test -x /usr/bin/lsb_release && /usr/bin/lsb_release --short --id 2>/dev/null)" in
-    "Ubuntu")
-	SUFFIX="0ubuntu1"
-    ;;
-esac
+if which dpkg-vendor >/dev/null 2>&1; then
+    case "$(dpkg-vendor --query Vendor 2>/dev/null)" in
+	"Ubuntu")
+	    SUFFIX="0ubuntu1"
+	    ;;
+    esac
+fi
 
 if [ "$1" = "--no-conf" -o "$1" = "--noconf" ]; then
     shift
@@ -264,7 +265,8 @@ if [ "$PATCH" ]; then
 	    case "$PATCH" in
 		*.gz)  CATPATCH="zcat $PATCH"; X=${X%.gz};;
 		*.bz2) CATPATCH="bzcat $PATCH"; X=${X%.bz2};;
-		*.lzma) CATPATCH="lzcat $PATCH"; X=${X%.lzma};;
+		*.lzma) CATPATCH="xz -F lzma -dc $PATCH"; X=${X%.lzma};;
+		*.xz) CATPATCH="xzcat $PATCH"; X=${X%.xz};;
 		*)     CATPATCH="cat $PATCH";;
 	    esac
 	    ;;
@@ -292,13 +294,22 @@ if [ "$PATCH" ]; then
 		    ;;
 		*.lzma)
 		    if [ -r "$OPWD/$PATCH" ]; then
-			CATPATCH="lzcat $OPWD/$PATCH"
+			CATPATCH="xz -F lzma -dc $OPWD/$PATCH"
 		    else
-			CATPATCH="lzcat ../$PATCH"
+			CATPATCH="xz -F lzma -dc ../$PATCH"
 		    fi
 		    X=${X%.lzma}
 		    ;;
-		*)    if [ -r "$OPWD/$PATCH" ]; then
+		*.xz)
+		    if [ -r "$OPWD/$PATCH" ]; then
+			CATPATCH="xzcat $OPWD/$PATCH"
+		    else
+			CATPATCH="xzcat ../$PATCH"
+		    fi
+		    X=${X%.xz}
+		    ;;
+		*)
+		    if [ -r "$OPWD/$PATCH" ]; then
 			CATPATCH="cat $OPWD/$PATCH"
 		    else
 			CATPATCH="cat ../$PATCH"
@@ -357,8 +368,11 @@ if [ "$PATCH" ]; then
     elif [ -r "../${PACKAGE}_$UVERSION.orig.tar.lzma" ]; then
 	OLDARCHIVE="${PACKAGE}_$UVERSION.orig.tar.lzma"
 	OLDARCHIVETYPE=lzma
+    elif [ -r "../${PACKAGE}_$UVERSION.orig.tar.xz" ]; then
+	OLDARCHIVE="${PACKAGE}_$UVERSION.orig.tar.xz"
+	OLDARCHIVETYPE=xz
     else
-	echo "$PROGNAME: can't find/read ${PACKAGE}_$UVERSION.orig.tar.{gz|bz2}" >&2
+	echo "$PROGNAME: can't find/read ${PACKAGE}_$UVERSION.orig.tar.{gz|bz2|lzma|xz}" >&2
 	echo "in the parent directory!" >&2
 	echo "Aborting...." >&2
 	exit 1
@@ -408,6 +422,12 @@ if [ "$PATCH" ]; then
 	    echo "aborting..." >&2
 	    exit 1
 	}
+    elif [ "$OLDARCHIVETYPE" = xz ]; then
+	tar --xz -xf ../$OLDARCHIVE || {
+	    echo "$PROGNAME: can't untar $OLDARCHIVE;" >&2
+	    echo "aborting..." >&2
+	    exit 1
+	}
     else
 	echo "$PROGNAME: internal error: unknown OLDARCHIVETYPE: $OLDARCHIVETYPE" >&2
 	exit 1
@@ -480,7 +500,7 @@ else
 		echo "$PROGNAME: cannot read archive file $ARCHIVE!  Aborting." >&2
 		exit 1
 	    fi
-	    
+
 	    ;;
     esac
 
@@ -495,13 +515,17 @@ else
 	                    TYPE=bz2 ;;
 	    *.orig.tar.lzma) X="${X%.orig.tar.lzma}"; UNPACK="tar --lzma -xf";
 	                    TYPE=lzma ;;
+	    *.orig.tar.xz) X="${X%.orig.tar.xz}"; UNPACK="tar --xz -xf";
+	                    TYPE=xz ;;
 	    *.tar.gz)  X="${X%.tar.gz}";  UNPACK="tar zxf"; TYPE=gz ;;
 	    *.tar.bz2) X="${X%.tar.bz2}"; UNPACK="tar --bzip -xf"; TYPE=bz2 ;;
 	    *.tar.lzma) X="${X%.tar.lzma}"; UNPACK="tar --lzma -xf"; TYPE=lzma ;;
+	    *.tar.xz)  X="${X%.tar.xz}";  UNPACK="tar --xz -xf"; TYPE=xz ;;
 	    *.tar.Z)   X="${X%.tar.Z}";   UNPACK="tar zxf"; TYPE="" ;;
 	    *.tgz)     X="${X%.tgz}";     UNPACK="tar zxf"; TYPE=gz ;;
 	    *.tar)     X="${X%.tar}";     UNPACK="tar xf";  TYPE="" ;;
 	    *.zip)     X="${X%.zip}";     UNPACK="unzip";   TYPE="" ;;
+	    *.7z)      X="${X%.7z}";      UNPACK="7z x";    TYPE="" ;;
 	    *)
 		echo "$PROGNAME: sorry: Unknown archive type" >&2
 		exit 1
@@ -571,12 +595,21 @@ else
 	echo "already exists in the parent dir;" >&2
 	echo "please check on the situation before trying $PROGNAME again." >&2
 	exit 1
+    elif [ -e "../${PACKAGE}_$SNEW_VERSION.orig.tar.xz" ] && \
+	[ "$(md5sum "${ARCHIVE_PATH}" | cut -d" " -f1)" != \
+	  "$(md5sum "../${PACKAGE}_$SNEW_VERSION.orig.tar.xz" | cut -d" " -f1)" ]
+    then
+	echo "$PROGNAME: a different ${PACKAGE}_$SNEW_VERSION.orig.tar.xz" >&2
+	echo "already exists in the parent dir;" >&2
+	echo "please check on the situation before trying $PROGNAME again." >&2
+	exit 1
     fi
 
     if [ $UUPDATE_PRISTINE = yes -a -n "$TYPE" -a \
 	! -e "../${PACKAGE}_$SNEW_VERSION.orig.tar.gz" -a \
 	! -e "../${PACKAGE}_$SNEW_VERSION.orig.tar.bz2" -a \
-	! -e "../${PACKAGE}_$SNEW_VERSION.orig.tar.lzma" ]; then
+	! -e "../${PACKAGE}_$SNEW_VERSION.orig.tar.lzma" -a \
+	! -e "../${PACKAGE}_$SNEW_VERSION.orig.tar.xz" ]; then
 	if [ "$UUPDATE_SYMLINK_ORIG" = yes ]; then
 	    echo "Symlinking to pristine source from ${PACKAGE}_$SNEW_VERSION.orig.tar.$TYPE..."
 	    case $ARCHIVE_PATH in
@@ -609,8 +642,15 @@ else
 		    cp "$ARCHIVE_PATH" "../${PACKAGE}_$SNEW_VERSION.orig.tar.lzma"
 		fi
 		;;
+	    xz)
+		if [ "$UUPDATE_SYMLINK_ORIG" = yes ]; then
+		    ln -s "$LINKARCHIVE" "../${PACKAGE}_$SNEW_VERSION.orig.tar.xz"
+		else
+		    cp "$ARCHIVE_PATH" "../${PACKAGE}_$SNEW_VERSION.orig.tar.xz"
+		fi
+		;;
 	    *)
-		echo "$PROGNAME: can't preserve pristine sources from non .tar.gz/.tar.bz2 upstream archive!" >&2
+		echo "$PROGNAME: can't preserve pristine sources from non .tar.{gz|bz2|lzma|xz} upstream archive!" >&2
 		echo "Continuing anyway..." >&2
 		;;
 	esac
@@ -646,6 +686,9 @@ else
 	# Otherwise, we put them into a new directory
 	mkdir $PACKAGE-$SNEW_VERSION
 	mv $TEMP_DIR/* $PACKAGE-$SNEW_VERSION
+	if ls $TEMP_DIR/.[!.]* >/dev/null 2>&1 ; then
+	    mv $TEMP_DIR/.[!.]* $PACKAGE-$SNEW_VERSION
+	fi
     fi
     rm -rf $TEMP_DIR
     cp -a $PACKAGE-$SNEW_VERSION $PACKAGE-$SNEW_VERSION.orig
@@ -662,7 +705,11 @@ else
     elif [ -r "../${PACKAGE}_$SVERSION.diff.lzma" ]; then
 	DIFF="../${PACKAGE}_$SVERSION.diff.lzma"
 	DIFFTYPE=diff
-	DIFFCAT=lzcat
+	DIFFCAT="xz -F lzma -dc"
+    elif [ -r "../${PACKAGE}_$SVERSION.diff.xz" ]; then
+	DIFF="../${PACKAGE}_$SVERSION.diff.xz"
+	DIFFTYPE=diff
+	DIFFCAT=xzcat
     elif [ -r "../${PACKAGE}_$SVERSION.debian.tar.gz" ]; then
 	DIFF="../${PACKAGE}_$SVERSION.debian.tar.gz"
 	DIFFTYPE=tar
@@ -675,6 +722,10 @@ else
 	DIFF="../${PACKAGE}_$SVERSION.debian.tar.lzma"
 	DIFFTYPE=tar
 	DIFFUNPACK="tar --lzma -xf"
+    elif [ -r "../${PACKAGE}_$SVERSION.debian.tar.xz" ]; then
+	DIFF="../${PACKAGE}_$SVERSION.debian.tar.xz"
+	DIFFTYPE=tar
+	DIFFUNPACK="tar --xz -xf"
     fi
 
     if [ "$DIFFTYPE" = diff ]; then
@@ -706,7 +757,7 @@ else
 		    echo "Please check that the diff is applied correctly." >&2
 		    echo "(This program will use the pristine upstream version and save the old .diff.gz" >&2
 		    echo "version as $file.debdiff .)" >&2
-		    
+
 		    if [ -e "$file.upstream" -o -e "$file.debdiff" ]; then
 			FILEEXISTERR=1
 		    fi
@@ -749,7 +800,7 @@ else
 
     elif [ "$DIFFTYPE" = tar ]; then
 	if [ -d debian ]; then
-	    echo "$PROGNAME warning: using a debian.tar.{gz|bz2|lzma} file in old Debian source," >&@
+	    echo "$PROGNAME warning: using a debian.tar.{gz|bz2|lzma|xz} file in old Debian source," >&2
 	    echo "but upstream also contains a debian/ directory!" >&2
 	    if [ -e "debian.upstream" ]; then
 		echo "Please apply the diff by hand and take care with this." >&2
@@ -761,12 +812,12 @@ else
 	fi
 	if $DIFFUNPACK $DIFF; then
 	    echo "Unpacking the debian/ directory from version $VERSION worked fine."
-	else 
+	else
 	    echo "$PROGNAME: failed to unpack the debian/ directory from version $VERSION!" >&2
 	    exit 1
 	fi
     else
-	echo "$PROGNAME: could not find diffs from version $VERSION to apply!" >&2
+	echo "$PROGNAME: could not find {diff|debian.tar}.{gz|bz2|lzma|xz} from version $VERSION to apply!" >&2
 	exit 1
     fi
     chmod a+x debian/rules
